function [p,L,U]=pLU_Cpt(A)
% permuted LU factorization for 3D-array
%   P*A=L*U
% p is the index of permutations for each matrices.
% One can use P=fc_linalg3D.fact.PermMatFromArray(p) to build the
% N-by-d-by-d 3D-array permutation matrices.
  assert( fc_linalg3D.is3Darray(A) )
  [N,m,n]=size(A);
  assert( m==n )
  L=fc_linalg3D.eye(N,m);
  U=A;
  p=repmat(1:m,N,1); % N-by-m permutations indices
  for k=1:N
    for i=1:m
      [~,idx]=max(abs(U(k,i:end,i)));
      mu=idx(1)+i-1;
      if i~=mu
        U(k,[i,mu],i:end)=U(k,[mu,i],i:end);
        L(k,[i,mu],1:i-1)=L(k,[mu,i],1:i-1);
        p(k,[i,mu])=p(k,[mu,i]);
      end
      for j=i+1:m
        L(k,j,i)=U(k,j,i)/U(k,i,i);
        U(k,j,i:end)=U(k,j,i:end)-L(k,j,i)*U(k,i,i:end);
      end
    end  
  end  
  if nargout==1, p={p,L,U};end
end
